import unittest

from stellanow_api_internals.datatypes.workflow_mgmt import (
    DAGAggregationNodeData,
    DAGAwaitNodeData,
    DAGNodeEmpty,
    DAGNodeWithCondition,
    DAGNotificationNodeData,
    StellaWorkflowDAG,
    StellaWorkflowDAGEdge,
    StellaWorkflowDAGNodes,
)


class TestParseDag(unittest.TestCase):
    def test_parse_workflow_dag(self):
        config_data = {
            "id": "2afa8461-19d2-4807-a89c-87639e6d4247",
            "workflowId": "955512e1-b40d-4981-8657-df508ccc5b0e",
            "commitMessage": "",
            "versionNumber": 7,
            "isLatest": True,
            "isPublished": False,
            "createdAt": "2024-08-13T10:23:33.848233Z",
            "structure": {
                "nodes": [
                    {
                        "type": "filter",
                        "id": "c09a8800-077b-4a26-aa01-c330ace78ba0",
                        "position": {"x": 960, "y": 60},
                        "data": {
                            "displayInfo": {"title": "FilterAirState", "description": ""},
                            "config": {
                                "id": "7018fff6-114b-470e-98c2-5505eb448570",
                                "conditions": [
                                    {
                                        "id": "01c1e650-929c-4722-bbe4-f8b4e8daed0a",
                                        "left": {
                                            "id": "6a0a5f4e-6200-4ab3-b7bf-16057dd6256d",
                                            "fieldName": "eventTypeDefinitionId",
                                        },
                                        "right": {
                                            "type": "ExplicitType",
                                            "id": "0bd4120b-8940-4ce3-bee3-4d8de39a9e40",
                                            "value": "air_quality_state",
                                            "multiValue": False,
                                        },
                                        "condition": "EQ",
                                        "negate": False,
                                        "fieldType": {"value": "String"},
                                    }
                                ],
                                "operator": "AND",
                            },
                            "isStateful": False,
                        },
                    },
                    {
                        "type": "notify",
                        "id": "57b80f86-83c7-46c0-8159-8ccd0f887638",
                        "position": {"x": 960, "y": 220},
                        "data": {
                            "displayInfo": {"title": "TestDestination"},
                            "config": {
                                "id": "32c54d47-ded2-4785-aadd-7ab13cc0c116",
                                "channel": "5d50656f-e5c7-4464-84b5-86f283e59bf2",
                                "fields": [
                                    {
                                        "targetFieldName": "name",
                                        "source": {
                                            "type": "EventType",
                                            "id": "db096e59-cf14-43fb-bdeb-43a1cc5fe497",
                                            "fieldId": "1a9b7af0-3e8d-4144-8c71-71eb2505d140",
                                            "multiValue": False,
                                            "eventName": "air_quality_state",
                                            "fieldName": "ijp.name",
                                        },
                                    }
                                ],
                            },
                            "isStateful": True,
                        },
                    },
                    {
                        "type": "propagateToSource",
                        "id": "504805b9-2cf8-4c2f-9f6c-b4469b1399ac",
                        "position": {"x": 960, "y": 380},
                        "data": {
                            "displayInfo": {"title": "Propagate to Source"},
                            "config": {"id": "5e6add30-db6a-471a-a2a1-316dc2c7a75f"},
                            "isStateful": False,
                        },
                    },
                    {
                        "type": "await",
                        "id": "await_node_123",
                        "position": {"x": 1000, "y": 400},
                        "data": {
                            "displayInfo": {"title": "Await Event", "description": "Waiting for event"},
                            "config": {
                                "id": "await_config_123",
                                "eventName": "some_event_name",
                                "waitTime": 120,
                            },
                            "isStateful": False,
                        },
                    },
                    {
                        "type": "aggregate",
                        "id": "aggregate_node_123",
                        "position": {"x": 1100, "y": 500},
                        "data": {
                            "displayInfo": {"title": "Aggregate Data"},
                            "config": {
                                "id": "aggregate_config_123",
                                "aggregated": {
                                    "id": "field_123",
                                    "fieldId": "field_456",
                                    "eventName": "event_name",
                                    "fieldName": "field_name",
                                    "multiValue": False,
                                },
                                "source": {
                                    "type": "EventType",
                                    "id": "source_123",
                                    "fieldId": "source_field_456",
                                    "multiValue": False,
                                    "eventName": "source_event",
                                    "fieldName": "source_field",
                                },
                                "interval": "1h",
                                "intervalStart": "2024-10-10T00:00:00Z",
                                "intervalLength": 60,
                            },
                            "isStateful": True,
                        },
                    },
                ],
                "edges": [
                    {
                        "id": "edge_1",
                        "source": "c09a8800-077b-4a26-aa01-c330ace78ba0",
                        "target": "57b80f86-83c7-46c0-8159-8ccd0f887638",
                    },
                    {
                        "id": "edge_2",
                        "source": "57b80f86-83c7-46c0-8159-8ccd0f887638",
                        "target": "504805b9-2cf8-4c2f-9f6c-b4469b1399ac",
                    },
                ],
            },
        }
        parsed_dag = StellaWorkflowDAG(**config_data)

        self.assertIsInstance(parsed_dag, StellaWorkflowDAG)
        self.assertEqual(len(parsed_dag.structure.nodes.root), 5)

        nodes_by_type = {node.type: node for node in parsed_dag.structure.nodes.root}

        filter_node = nodes_by_type.get("filter")
        self.assertIsNotNone(filter_node)
        self.assertEqual(filter_node.data.displayInfo.title, "FilterAirState")
        self.assertIsInstance(filter_node.data.config, DAGNodeWithCondition)

        notify_node = nodes_by_type.get("notify")
        self.assertIsNotNone(notify_node)
        self.assertEqual(notify_node.data.displayInfo.title, "TestDestination")
        self.assertIsInstance(notify_node.data.config, DAGNotificationNodeData)

        propagate_node = nodes_by_type.get("propagateToSource")
        self.assertIsNotNone(propagate_node)
        self.assertEqual(propagate_node.data.displayInfo.title, "Propagate to Source")
        self.assertIsInstance(propagate_node.data.config, DAGNodeEmpty)

        await_node = nodes_by_type.get("await")
        self.assertIsNotNone(await_node)
        self.assertEqual(await_node.data.displayInfo.title, "Await Event")
        self.assertIsInstance(await_node.data.config, DAGAwaitNodeData)

        aggregate_node = nodes_by_type.get("aggregate")
        self.assertIsNotNone(aggregate_node)
        self.assertEqual(aggregate_node.data.displayInfo.title, "Aggregate Data")
        self.assertIsInstance(aggregate_node.data.config, DAGAggregationNodeData)

        for edge in parsed_dag.structure.edges.root:
            self.assertIsInstance(edge, StellaWorkflowDAGEdge)

        self.assertEqual(parsed_dag.id, config_data["id"])
        self.assertEqual(parsed_dag.workflowId, config_data["workflowId"])
        self.assertEqual(parsed_dag.versionNumber, config_data["versionNumber"])

    def test_parse_dag_node_filter(self):
        node_data = {
            "type": "filter",
            "id": "c09a8800-077b-4a26-aa01-c330ace78ba0",
            "position": {"x": 960, "y": 60},
            "data": {
                "displayInfo": {"title": "FilterAirState", "description": ""},
                "config": {
                    "id": "7018fff6-114b-470e-98c2-5505eb448570",
                    "conditions": [
                        {
                            "id": "01c1e650-929c-4722-bbe4-f8b4e8daed0a",
                            "left": {
                                "id": "6a0a5f4e-6200-4ab3-b7bf-16057dd6256d",
                                "fieldName": "eventTypeDefinitionId",
                            },
                            "right": {
                                "type": "ExplicitType",
                                "id": "0bd4120b-8940-4ce3-bee3-4d8de39a9e40",
                                "value": "air_quality_state",
                                "multiValue": False,
                            },
                            "condition": "EQ",
                            "negate": False,
                            "fieldType": {"value": "String"},
                        }
                    ],
                    "operator": "AND",
                },
                "isStateful": False,
            },
        }
        parsed_node = StellaWorkflowDAGNodes(**node_data)

        self.assertIsInstance(parsed_node, StellaWorkflowDAGNodes)
        self.assertIsInstance(parsed_node.data.config, DAGNodeWithCondition)
        self.assertEqual(parsed_node.id, node_data["id"])
        self.assertEqual(parsed_node.data.config.operator, node_data["data"]["config"]["operator"])



from http import HTTPStatus
from unittest.mock import patch

import httpx
import pytest

from stellanow_api_internals.clients.workflow_manager_client import WorkflowManagerClient


@pytest.fixture
def mock_workflow_client():
    """Factory fixture that requires a custom response."""

    def create_mock_client(custom_response):
        if not custom_response:
            raise ValueError("You must provide a custom response")

        def mock_transport(request):
            return httpx.Response(HTTPStatus.OK, json=custom_response)

        transport = httpx.MockTransport(mock_transport)
        client = httpx.Client(transport=transport)

        with patch.object(WorkflowManagerClient, "authenticate", return_value=None):
            workflow_client = WorkflowManagerClient(
                base_url="https://mockapi.com",
                username="user",
                password="pass",
                organization_id="org_id",
                project_id="8db9f3bf-bea1-46dc-9364-e9dc4f2fa01f",
                client=client,
            )
            workflow_client.auth_token = "mocked_token"  # Mock token
            return workflow_client

    return create_mock_client



"""
Copyright (C) 2022-2024 Stella Technologies (UK) Limited.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
"""

from typing import List, Optional

from pydantic import BaseModel, ConfigDict, RootModel

from stellanow_api_internals.datatypes.core import StellaFormattedDateTime


class MessageTemplate(BaseModel):
    summary: str
    body: str


class StellaNotificationService(StellaFormattedDateTime):
    serviceId: str
    name: str
    description: Optional[str] = None
    isActive: bool
    serviceType: str
    url: str
    auth: Optional[str] = None

    model_config = ConfigDict(extra="ignore")


class StellaNotificationServiceList(RootModel):
    root: List[StellaNotificationService]


class StellaNotificationServiceDetailed(StellaNotificationService):
    pass


class StellaNotificationServiceDetailedList(RootModel):
    root: List[StellaNotificationServiceDetailed]


class PlaceholderResponse(StellaFormattedDateTime):
    placeholderId: str
    name: str
    type: str


class PlaceholderResponseList(RootModel):
    root: List[PlaceholderResponse]


class PlaceholderCreate(BaseModel):
    name: str
    type: str


class PlaceholderCreateList(RootModel):
    root: List[PlaceholderCreate]


class StellaChannel(StellaFormattedDateTime):
    name: str
    description: Optional[str] = None
    isActive: bool
    realmId: str
    projectId: str
    channelId: str
    placeholders: Optional[PlaceholderResponseList] = None


class StellaChannelCreate(BaseModel):
    name: str
    description: Optional[str] = None
    isActive: bool
    placeholders: Optional[PlaceholderCreateList]

    model_config = ConfigDict(extra="ignore")


class StellaDestinationChannel(StellaFormattedDateTime):
    name: str
    description: Optional[str] = None
    isActive: bool
    destinationId: str
    urlTemplate: Optional[str] = None
    requestType: Optional[str] = None


class StellaDestinationChannelList(RootModel):
    root: List[StellaDestinationChannel]


class StellaChannelDetailed(StellaChannel):
    destinationChannel: Optional[StellaDestinationChannelList] = None


class StellaChannelDetailedList(RootModel):
    root: List[StellaChannelDetailed]


class StellaDestination(StellaFormattedDateTime):
    name: str
    description: Optional[str] = None
    isActive: bool
    destinationId: str
    urlTemplate: Optional[str] = None
    requestType: Optional[str] = None
    messageTemplate: Optional[MessageTemplate] = None
    bodyTemplate: Optional[str] = None
    jiraProject: Optional[str] = None
    jiraIssueType: Optional[int] = None


class StellaDestinationList(RootModel):
    root: List[StellaDestination]


class StellaDestinationDetailed(StellaDestination):
    service: StellaNotificationServiceDetailed


class StellaDestinationDetailedList(RootModel):
    root: List[StellaDestinationDetailed]


class StellaDestinationCreate(BaseModel):
    name: str
    description: Optional[str] = None
    isActive: bool
    urlTemplate: Optional[str] = None
    requestType: Optional[str] = None
    messageTemplate: Optional[MessageTemplate] = None
    bodyTemplate: Optional[str] = None
    jiraProject: Optional[str] = None
    jiraIssueType: Optional[int] = None
    serviceId: str

    model_config = ConfigDict(extra="ignore")



import unittest

from stellanow_api_internals.datatypes.workflow_mgmt import (
    StellaBaseFieldType,
    StellaEntityDetailed,
    StellaEntityField,
    StellaShortEvent,
)


class TestStellaEntityDetailedSerialization(unittest.TestCase):

    def test_stella_entity_detailed_serialization(self):
        json_data = {
            "details": {
                "projectId": "52b212f7-3da9-47eb-a2a8-adc9248dba74",
                "id": "61d16181-12f0-4f7a-a702-e0e6bb06302a",
                "name": "Metro",
                "description": "",
                "isActive": True,
                "createdAt": "2024-03-11T10:48:23.288147Z",
                "updatedAt": "2024-03-11T10:48:56.094698Z",
                "fields": [],
                "events": [
                    {"id": "e716433e-cc49-49ff-a1ba-b8a131053a12", "name": "metro_state"},
                    {"id": "1f6d835e-2f01-468f-9501-04a6f7509389", "name": "test_event_sdk"},
                ],
            },
            "status": "ok",
        }

        expected_entity = StellaEntityDetailed(
            projectId="52b212f7-3da9-47eb-a2a8-adc9248dba74",
            id="61d16181-12f0-4f7a-a702-e0e6bb06302a",
            name="Metro",
            description="",
            isActive=True,
            createdAt="2024-03-11T10:48:23.288147Z",
            updatedAt="2024-03-11T10:48:56.094698Z",
            fields=[],
            events=[
                StellaShortEvent(id="e716433e-cc49-49ff-a1ba-b8a131053a12", name="metro_state"),
                StellaShortEvent(id="1f6d835e-2f01-468f-9501-04a6f7509389", name="test_event_sdk"),
            ],
        )

        parsed_entity = StellaEntityDetailed(
            projectId=json_data["details"]["projectId"],
            id=json_data["details"]["id"],
            name=json_data["details"]["name"],
            description=json_data["details"]["description"],
            isActive=json_data["details"]["isActive"],
            createdAt=json_data["details"]["createdAt"],
            updatedAt=json_data["details"]["updatedAt"],
            fields=json_data["details"]["fields"],
            events=[StellaShortEvent(**event) for event in json_data["details"]["events"]],
        )

        self.assertEqual(parsed_entity.dict(), expected_entity.dict())

    def test_stella_entity_detailed_parsing(self):
        # Input data
        input_data = {
            "projectId": "52b212f7-3da9-47eb-a2a8-adc9248dba74",
            "id": "f4ffebe0-be4c-40ea-9e46-5fc6b7364158",
            "name": "AirQuality",
            "description": "AirQuality",
            "isActive": True,
            "createdAt": "2024-03-01T13:31:57.357604Z",
            "updatedAt": "2024-03-01T14:52:42.334055Z",
            "fields": [
                {
                    "id": "81b6d462-cac3-4fc9-89f4-20928acce88d",
                    "name": "locationId",
                    "fieldType": {"value": "Integer"},
                    "propagatedFrom": [],
                }
            ],
            "events": [
                {"id": "c36508cf-4c52-4454-81c8-1bd3981d4264", "name": "air_quality_state"},
                {"id": "1f6d835e-2f01-468f-9501-04a6f7509389", "name": "test_event_sdk"},
            ],
        }

        # Convert input data to StellaEntityDetailed instance
        entity_detailed = StellaEntityDetailed(
            id=input_data["id"],
            name=input_data["name"],
            description=input_data["description"],
            projectId=input_data["projectId"],
            isActive=input_data["isActive"],
            createdAt=input_data["createdAt"],
            updatedAt=input_data["updatedAt"],
            fields=[
                StellaEntityField(
                    id=field["id"],
                    name=field["name"],
                    fieldType=StellaBaseFieldType(**field["fieldType"]),
                    propagatedFrom=field["propagatedFrom"],
                )
                for field in input_data["fields"]
            ],
            events=[StellaShortEvent(**event) for event in input_data["events"]],
        )

        # Assertions
        self.assertEqual(entity_detailed.id, "f4ffebe0-be4c-40ea-9e46-5fc6b7364158")
        self.assertEqual(entity_detailed.name, "AirQuality")
        self.assertEqual(entity_detailed.description, "AirQuality")
        self.assertEqual(entity_detailed.projectId, "52b212f7-3da9-47eb-a2a8-adc9248dba74")
        self.assertTrue(entity_detailed.isActive)
        self.assertEqual(entity_detailed.createdAt, "2024-03-01 13:31:57")
        self.assertEqual(entity_detailed.updatedAt, "2024-03-01 14:52:42")

        self.assertEqual(len(entity_detailed.fields), 1)
        self.assertEqual(entity_detailed.fields[0].id, "81b6d462-cac3-4fc9-89f4-20928acce88d")
        self.assertEqual(entity_detailed.fields[0].name, "locationId")
        self.assertEqual(entity_detailed.fields[0].fieldType.value, "Integer")
        self.assertEqual(entity_detailed.fields[0].propagatedFrom, [])

        self.assertEqual(len(entity_detailed.events), 2)
        self.assertEqual(entity_detailed.events[0].id, "c36508cf-4c52-4454-81c8-1bd3981d4264")
        self.assertEqual(entity_detailed.events[1].id, "1f6d835e-2f01-468f-9501-04a6f7509389")
        self.assertEqual(entity_detailed.events[1].name, "test_event_sdk")



import json
import unittest
from json import loads

from stellanow_api_internals.datatypes.workflow_mgmt import (
    DAGNodeConfig,
    DAGNodeEventTypeFieldDefinition,
    DAGNotificationField,
    DAGNotificationNodeData,
)


class TestDAGNodeConfigSerialization(unittest.TestCase):

    def test_dag_node_config_serialization(self):
        # Create a sample DAGNotificationNodeData object
        notification_data = DAGNotificationNodeData(
            id="32c54d47-ded2-4785-aadd-7ab13cc0c116",
            channel="5d50656f-e5c7-4464-84b5-86f283e59bf2",
            fields=[
                DAGNotificationField(
                    targetFieldName="name",
                    source=DAGNodeEventTypeFieldDefinition(
                        id="db096e59-cf14-43fb-bdeb-43a1cc5fe497",
                        type="EventType",
                        fieldId="1a9b7af0-3e8d-4144-8c71-71eb2505d140",
                        eventName="air_quality_state",
                        fieldName="ijp.name",
                        multiValue=False,
                    ),
                )
            ],
        )

        config_dict = DAGNodeConfig(
            id="32c54d47-ded2-4785-aadd-7ab13cc0c116", notificationData=notification_data.model_dump()
        ).to_dict()

        expected_dict = {
            "id": "32c54d47-ded2-4785-aadd-7ab13cc0c116",
            "channel": "5d50656f-e5c7-4464-84b5-86f283e59bf2",
            "fields": [
                {
                    "targetFieldName": "name",
                    "source": {
                        "type": "EventType",
                        "id": "db096e59-cf14-43fb-bdeb-43a1cc5fe497",
                        "fieldId": "1a9b7af0-3e8d-4144-8c71-71eb2505d140",
                        "multiValue": False,
                        "eventName": "air_quality_state",
                        "fieldName": "ijp.name",
                    },
                }
            ],
        }

        self.assertEqual(config_dict, expected_dict)

        config_json = json.dumps(config_dict, indent=4)
        expected_json = json.dumps(expected_dict, indent=4)

        self.assertEqual(config_json, expected_json)

    def test_parse_dag_node_config(self):
        # Simulating a JSON payload from the system
        config_data = {
            "id": "7018fff6-114b-470e-98c2-5505eb448570",
            "filterData": {
                "id": "7018fff6-114b-470e-98c2-5505eb448570",
                "conditions": [
                    {
                        "id": "01c1e650-929c-4722-bbe4-f8b4e8daed0a",
                        "left": {"id": "6a0a5f4e-6200-4ab3-b7bf-16057dd6256d", "fieldName": "eventTypeDefinitionId"},
                        "right": {
                            "type": "ExplicitType",
                            "id": "0bd4120b-8940-4ce3-bee3-4d8de39a9e40",
                            "value": "air_quality_state",
                            "multiValue": False,
                        },
                        "condition": "EQ",
                        "negate": False,
                        "fieldType": {"value": "String"},
                    }
                ],
                "operator": "AND",
            },
        }

        # Parse the config and flatten the fields
        config_dict = DAGNodeConfig(id=config_data["id"], filterData=config_data["filterData"]).to_dict()

        # Ensure the JSON output is flattened correctly
        expected_dict = {
            "id": "7018fff6-114b-470e-98c2-5505eb448570",
            "conditions": [
                {
                    "id": "01c1e650-929c-4722-bbe4-f8b4e8daed0a",
                    "left": {"id": "6a0a5f4e-6200-4ab3-b7bf-16057dd6256d", "fieldName": "eventTypeDefinitionId"},
                    "right": {
                        "type": "ExplicitType",
                        "id": "0bd4120b-8940-4ce3-bee3-4d8de39a9e40",
                        "value": "air_quality_state",
                        "multiValue": False,
                    },
                    "condition": "EQ",
                    "negate": False,
                    "fieldType": {"value": "String"},
                }
            ],
            "operator": "AND",
        }

        self.assertEqual(config_dict, expected_dict)

        config_json = json.dumps(config_dict, indent=4)
        expected_json = json.dumps(expected_dict, indent=4)

        self.assertEqual(loads(expected_json), loads(config_json))



"""
Copyright (C) 2022-2024 Stella Technologies (UK) Limited.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
"""

from typing import Dict, List, Optional

import httpx

from stellanow_api_internals.clients.base_api_client import StellanowBaseAPIClient
from stellanow_api_internals.core.enums import CreateOrUpdateTypes, FilterIncludeArchived, FilterIncludeInactive
from stellanow_api_internals.datatypes.workflow_mgmt import (
    StellaEntity,
    StellaEntityCreate,
    StellaEntityDetailed,
    StellaEntityUpdate,
    StellaEvent,
    StellaEventCreate,
    StellaEventDetailed,
    StellaModel,
    StellaModelCreate,
    StellaModelDetailed,
    StellaProject,
    StellaProjectCreate,
    StellaProjectDetailed,
    StellaWorkflow,
    StellaWorkflowCreate,
    StellaWorkflowDAG,
    StellaWorkflowDagCreate,
    StellaWorkflowDetailed,
)

WORKFLOW_MANAGEMENT_API = "/workflow-management/projects/"


class WorkflowManagerClient(StellanowBaseAPIClient):
    def __init__(
        self,
        base_url: str,
        username: str,
        password: str,
        organization_id: str,
        project_id: Optional[str] = None,
        client: Optional[httpx.Client] = None,
    ) -> None:
        super().__init__(base_url, username, password, organization_id, client)
        self.project_id = project_id or ""

    @property
    def base_path(self) -> str:
        return WORKFLOW_MANAGEMENT_API

    @property
    def _projects_url(self):
        return f"{self.base_url}{WORKFLOW_MANAGEMENT_API}"

    @property
    def _project_url(self):
        return self._build_url_project_required("")

    @property
    def _events_url(self):
        return self._build_url_project_required("/events")

    @property
    def _event_url(self):
        return self._build_url_project_required("/events/{eventId}")

    @property
    def _entities_url(self):
        return self._build_url_project_required("/entities")

    @property
    def _entity_url(self):
        return self._build_url_project_required("/entities/{entityId}")

    @property
    def _models_url(self):
        return self._build_url_project_required("/models")

    @property
    def _model_url(self):
        return self._build_url_project_required("/models/{modelId}")

    @property
    def _workflows_url(self):
        return self._build_url_project_required("/workflows")

    @property
    def _workflow_url(self):
        return self._build_url_project_required("/workflows/{workflowId}")

    @property
    def _workflow_dag_latest_url(self):
        return self._build_url_project_required("/workflows/{workflowId}/dag/latest")

    @property
    def _workflow_dag_version_url(self):
        return self._build_url_project_required("/workflows/{workflowId}/dag/versions/{versionId}")

    def create_project(self, project_data: Dict) -> StellaProjectDetailed:
        return self._create_or_update_resource(
            url=self._projects_url,
            data_model_class=StellaProjectCreate,
            return_class=StellaProjectDetailed,
            resource_data=project_data,
        )

    def create_model(self, model_data: Dict) -> StellaModelDetailed:
        return self._create_or_update_resource(
            url=self._models_url,
            data_model_class=StellaModelCreate,
            return_class=StellaModelDetailed,
            resource_data=model_data,
        )

    def create_event(self, event_data: Dict) -> StellaEventDetailed:
        return self._create_or_update_resource(
            url=self._events_url,
            data_model_class=StellaEventCreate,
            return_class=StellaEventDetailed,
            resource_data=event_data,
        )

    def create_entity(self, entity_data: Dict) -> StellaEntityDetailed:
        return self._create_or_update_resource(
            url=self._entities_url,
            data_model_class=StellaEntityCreate,
            return_class=StellaEntityDetailed,
            resource_data=entity_data,
        )

    def update_entity(self, entity_data: Dict, entity_id: str) -> StellaEntityDetailed:
        return self._create_or_update_resource(
            url=self._entity_url.format(entityId=entity_id),
            data_model_class=StellaEntityUpdate,
            return_class=StellaEntityDetailed,
            resource_data=entity_data,
            method=CreateOrUpdateTypes.PATCH,
        )

    def create_workflow(self, workflow_data: Dict) -> StellaWorkflowDetailed:
        return self._create_or_update_resource(
            url=self._workflows_url,
            data_model_class=StellaWorkflowCreate,
            return_class=StellaWorkflowDetailed,
            resource_data=workflow_data,
        )

    def create_workflow_dag(self, workflow_id: str, dag_data: Dict) -> StellaWorkflowDAG:
        return self._create_or_update_resource(
            url=f"{self._workflows_url}/{workflow_id}/dag",
            data_model_class=StellaWorkflowDagCreate,
            return_class=StellaWorkflowDAG,
            resource_data=dag_data,
        )

    def get_projects(
        self,
        page: int = 1,
        page_size: int = 20,
        include_archived: Optional[bool] = None,
        search: Optional[str] = None,
        sorting: Optional[str] = "projects:created:asc",
    ) -> List[StellaProject]:
        filter_query = FilterIncludeArchived.INCLUDE_ARCHIVED.value if include_archived else None

        return self._get_list_resource(
            url=self._projects_url,
            result_class=StellaProject,
            page=page,
            page_size=page_size,
            filter=filter_query,
            search=search,
            sorting=sorting,
        )

    def get_project_details(self) -> StellaProjectDetailed:
        return self._get_resource(url=self._project_url, result_class=StellaProjectDetailed)

    def get_events(
        self,
        page: int = 1,
        page_size: int = 20,
        include_inactive: Optional[bool] = None,
        search: Optional[str] = None,
        sorting: Optional[str] = "events:created:asc",
    ) -> List[StellaEvent]:
        filter_query = FilterIncludeInactive.INCLUDE_INACTIVE.value if include_inactive else None

        return self._get_list_resource(
            url=self._events_url,
            result_class=StellaEvent,
            page=page,
            page_size=page_size,
            filter=filter_query,
            search=search,
            sorting=sorting,
        )

    def get_event_details(self, event_id: str) -> StellaEventDetailed:
        return self._get_resource(url=self._event_url.format(eventId=event_id), result_class=StellaEventDetailed)

    def get_entities(
        self,
        page: int = 1,
        page_size: int = 20,
        include_inactive: Optional[bool] = None,
        search: Optional[str] = None,
        sorting: Optional[str] = "entities:created:asc",
    ) -> List[StellaEntity]:
        filter_query = FilterIncludeInactive.INCLUDE_INACTIVE.value if include_inactive else None

        return self._get_list_resource(
            url=self._entities_url,
            result_class=StellaEntity,
            page=page,
            page_size=page_size,
            filter=filter_query,
            search=search,
            sorting=sorting,
        )

    def get_entity_details(self, entity_id: str) -> StellaEntityDetailed:
        return self._get_resource(url=self._entity_url.format(entityId=entity_id), result_class=StellaEntityDetailed)

    def get_models(
        self,
        page: int = 1,
        page_size: int = 20,
        search: Optional[str] = None,
        sorting: Optional[str] = None,
    ) -> List[StellaModel]:

        return self._get_list_resource(
            url=self._models_url,
            result_class=StellaModel,
            page=page,
            page_size=page_size,
            search=search,
            sorting=sorting,
        )

    def get_model_details(self, model_id: str) -> StellaModelDetailed:
        return self._get_resource(url=self._model_url.format(modelId=model_id), result_class=StellaModelDetailed)

    def get_workflows(
        self,
        page: int = 1,
        page_size: int = 20,
        include_inactive: Optional[bool] = None,
        search: Optional[str] = None,
        sorting: Optional[str] = "workflows:created:asc",
    ) -> List[StellaWorkflow]:
        filter_query = FilterIncludeInactive.INCLUDE_INACTIVE.value if include_inactive else None

        return self._get_list_resource(
            url=self._workflows_url,
            result_class=StellaWorkflow,
            page=page,
            page_size=page_size,
            filter=filter_query,
            search=search,
            sorting=sorting,
        )

    def get_workflow_details(self, workflow_id: str) -> StellaWorkflowDetailed:
        return self._get_resource(
            url=self._workflow_url.format(workflowId=workflow_id),
            result_class=StellaWorkflowDetailed,
        )

    def get_latest_workflow_dag(self, workflow_id: str) -> StellaWorkflowDAG:
        details = self._get_resource(
            url=self._workflow_dag_latest_url.format(workflowId=workflow_id),
            result_class=StellaWorkflowDAG,
        )
        return details

    def get_workflow_dag_version(self, workflow_id: str, version_id: str) -> StellaWorkflowDAG:
        details = self._get_resource(
            url=self._workflow_dag_version_url.format(workflowId=workflow_id, versionId=version_id),
            result_class=StellaWorkflowDAG,
        )
        return details



import unittest
from datetime import datetime
from http import HTTPStatus
from unittest.mock import patch

import httpx

from stellanow_api_internals.auth.keycloak_auth import KeycloakAuth
from stellanow_api_internals.clients.base_api_client import StellanowBaseAPIClient


class TestStellanowBaseAPIClient(unittest.TestCase):

    def setUp(self):
        self.patcher = patch.object(
            KeycloakAuth,
            "get_token",
            return_value={"access_token": "mock_access_token", "refresh_token": "mock_refresh_token"},
        )
        self.mock_get_token = self.patcher.start()

        self.patcher_refresh = patch.object(
            KeycloakAuth,
            "refresh_token",
            return_value={"access_token": "mock_access_token", "refresh_token": "mock_refresh_token"},
        )
        self.mock_refresh_token = self.patcher_refresh.start()

    def tearDown(self):
        self.patcher.stop()
        self.patcher_refresh.stop()

    def create_mock_transport(self, json_data, status_code=HTTPStatus.OK):
        """Helper function to create a MockTransport with specified JSON response."""
        return httpx.MockTransport(lambda request: httpx.Response(status_code, json=json_data))

    def test_authenticate(self):
        transport = self.create_mock_transport(json_data={"key": "value"})
        client = StellanowBaseAPIClient(
            base_url="https://api.stellanow.com",
            username="test_user",
            password="test_password",
            organization_id="test_org",
            client=httpx.Client(transport=transport),  # Pass the mocked transport
        )

        client.authenticate()
        self.assertEqual(client.auth_token, "mock_access_token")
        self.assertEqual(client.refresh_token, "mock_refresh_token")

    def test_make_request_get(self):
        transport = self.create_mock_transport(json_data={"details": {"key": "value"}})
        client = StellanowBaseAPIClient(
            base_url="https://api.stellanow.com",
            username="test_user",
            password="test_password",
            organization_id="test_org",
            client=httpx.Client(transport=transport),  # Pass the mocked transport
        )

        response = client._make_request(url="https://api.stellanow.com/test", method="GET")
        self.assertEqual(response, {"key": "value"})

    def test_handle_dates(self):
        # Simulate response with date fields
        mock_data = {
            "details": {"createdAt": "2024-03-06T18:22:19.335921Z", "updatedAt": "2024-06-27T08:28:40.973269Z"}
        }
        transport = self.create_mock_transport(json_data=mock_data)
        client = StellanowBaseAPIClient(
            base_url="https://api.stellanow.com",
            username="test_user",
            password="test_password",
            organization_id="test_org",
            client=httpx.Client(transport=transport),  # Pass the mocked transport
        )

        details = client._make_request(url="https://api.stellanow.com/test", method="GET")

        # Convert the dates to the expected ISO format
        created_at = datetime.fromisoformat(details["createdAt"].replace("Z", "+00:00"))
        updated_at = datetime.fromisoformat(details["updatedAt"].replace("Z", "+00:00"))

        # Assert with the correct format (including milliseconds)
        self.assertEqual(created_at.isoformat(), "2024-03-06T18:22:19.335921+00:00")
        self.assertEqual(updated_at.isoformat(), "2024-06-27T08:28:40.973269+00:00")



"""
Copyright (C) 2022-2024 Stella Technologies (UK) Limited.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
"""

import json
from http import HTTPStatus
from typing import Any, Dict, Generic, List, Optional, Type, TypeVar

import httpx
from loguru import logger

from stellanow_api_internals.auth.keycloak_auth import KeycloakAuth
from stellanow_api_internals.core.enums import CreateOrUpdateTypes
from stellanow_api_internals.exceptions.api_exceptions import (
    StellaAPIBadRequestError,
    StellaAPIConflictError,
    StellaAPIForbiddenError,
    StellaAPINotFoundError,
    StellaAPIUnauthorisedError,
    StellaAPIWrongCredentialsError,
)

T = TypeVar("T")


class StellanowBaseAPIClient(Generic[T]):
    def __init__(
        self, base_url: str, username: str, password: str, organization_id: str, client: Optional[httpx.Client] = None
    ) -> None:
        self.base_url = base_url
        self.username = username
        self.password = password
        self.organization_id = organization_id
        self.auth_token = None
        self.refresh_token = None
        self.client = client or httpx.Client()

        self.keycloak = KeycloakAuth(
            server_url=self._auth_url, client_id="tools-cli", realm_name=self.organization_id, verify=True
        )

        self.authenticate()

    @property
    def _auth_url(self):
        return f"{self.base_url}/auth/"

    def _handle_response(self, response, url, method=None) -> None:
        """
        Handle the API response and raise appropriate exceptions for error status codes.

        :param response: The HTTP response object.
        :param url: The URL that was called.
        :param method: The HTTP method used (optional, for logging purposes).
        """
        try:
            details = response.json().get("details", {})
            if not isinstance(details, dict):
                details = {}
        except json.JSONDecodeError:
            details = {}

        status_code = response.status_code
        error_message = details.get("errorMessage", "No error details provided")
        request_info = f"{method or 'REQUEST'} to {url}"

        if status_code >= HTTPStatus.BAD_REQUEST:
            logger.error(f"API request failed with status {status_code} for {request_info}: {error_message}")

        match status_code:
            case HTTPStatus.BAD_REQUEST:
                raise StellaAPIBadRequestError(f"Bad Request: {error_message}")
            case HTTPStatus.UNAUTHORIZED:
                errors = details.get("errors", [])
                if not isinstance(errors, list):
                    errors = []

                for error in errors:
                    match error.get("errorCode"):
                        case "inactiveAuthToken":
                            self.auth_refresh()
                            return
                        case "wrongUsernameOrPassword":
                            raise StellaAPIWrongCredentialsError()
                        case _:
                            raise StellaAPIUnauthorisedError(f"Unauthorized: {error_message}")
                else:
                    response.raise_for_status()
            case HTTPStatus.FORBIDDEN:
                raise StellaAPIForbiddenError(f"Forbidden: {error_message}")
            case HTTPStatus.NOT_FOUND:
                raise StellaAPINotFoundError(f"Not Found: {error_message}")
            case HTTPStatus.CONFLICT:
                raise StellaAPIConflictError(f"Conflict: {error_message}")
            case _:
                response.raise_for_status()

    def authenticate(self):
        logger.info(f"Authenticating to the {self.__class__.__name__} API ... ")

        if self.refresh_token is not None:
            self.auth_refresh()
        else:
            self.auth_token = None
            self.refresh_token = None

            response = self.keycloak.get_token(self.username, self.password)

            self.auth_token = response.get("access_token")
            self.refresh_token = response.get("refresh_token")

        logger.info("Authentication Successful")

    def auth_refresh(self):
        if self.refresh_token is None:
            self.authenticate()
        else:
            logger.info("API Token Refreshing ...")

            refresh_token = self.refresh_token

            self.auth_token = None
            self.refresh_token = None

            response = self.keycloak.refresh_token(refresh_token)

            self.auth_token = response.get("access_token")
            self.refresh_token = response.get("refresh_token")

            logger.info("API Token Refresh Successful")

    def _make_request(
        self,
        url: str,
        method: Optional[str] = "GET",
        data: Optional[Dict[str, Any]] = None,
        params: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        headers = {"Authorization": f"Bearer {self.auth_token}"}
        method = method or "GET"
        response = self.client.request(method=method, url=url, headers=headers, json=data, params=params)
        self._handle_response(response=response, url=url, method=method)
        return response.json().get("details", {})

    def _create_or_update_resource(
        self,
        url: str,
        data_model_class: Type[Any],
        return_class: Type[Any],
        resource_data: Dict,
        method: Optional[CreateOrUpdateTypes] = CreateOrUpdateTypes.POST,
    ) -> Any:
        """
        Helper method to make requests (POST/PATCH) and validate that the response is a dict.

        :param url: The API endpoint URL.
        :param data_model_class: The Pydantic model class for the request data.
        :param return_class: The Pydantic model class for the response data.
        :param resource_data: The data to be sent in the request.
        :param method: The HTTP method ('POST' or 'PATCH').
        :return: An instance of the return_class containing the parsed response.
        """
        data_model_instance = data_model_class(**resource_data)
        details = self._make_request(url=url, method=method, data=data_model_instance.model_dump())

        if isinstance(details, List):
            raise TypeError(f"Expected a dict but got a list for {return_class.__name__} details.")

        return return_class(**details)

    def _get_resource(
        self,
        url: str,
        result_class: Type[T],
    ) -> T:
        """
        Fetch a single resource from the API.

        :param url: The API endpoint URL.
        :param result_class: The Pydantic model class for the expected result.
        :return: An instance of result_class.
        """
        details = self._make_request(url=url)

        if not isinstance(details, Dict):
            raise TypeError(f"Expected a dict but got {type(details)}")

        return result_class(**details)

    def _get_list_resource(
        self,
        url: str,
        result_class: Type[T],
        page: int = 1,
        page_size: int = 20,
        filter: Optional[str] = None,
        search: Optional[str] = None,
        sorting: Optional[str] = None,
    ) -> List[T]:
        """
        Fetch a list of resources (paginated) from the API.
        This method handles pagination and returns a list of resources.
        """
        params = self._build_query_params(page=page, page_size=page_size, filter=filter, search=search, sorting=sorting)
        details = self._make_request(url=url, params=params)

        return self._handle_paginated_results(
            initial_details=details,
            result_class=result_class,
            url=url,
            filter=filter,
            search=search,
            sorting=sorting,
        )

    def _handle_paginated_results(
        self,
        initial_details: Dict[str, Any],
        result_class: Type[T],
        url: str,
        filter: Optional[str] = None,
        search: Optional[str] = None,
        sorting: Optional[str] = None,
    ) -> List[T]:
        """
        Helper method to handle paginated results based on the initial response.

        :param initial_details: The initial details from the first paginated response.
        :param result_class: The Pydantic model class for the response data.
        :param url: The API endpoint URL.
        :param filter: An optional filter string.
        :param search: An optional search string.
        :param sorting: An optional sorting string.
        :return: A list of result_class instances.
        """
        results: List[T] = []

        page_results = initial_details.get("results", [])
        results.extend(result_class(**item) for item in page_results)

        page_number = initial_details.get("pageNumber", 1)
        number_of_pages = initial_details.get("numberOfPages", 1)
        page_size = initial_details.get("pageSize", 20)

        while page_number < number_of_pages:
            page_number += 1
            params = self._build_query_params(
                page=page_number, page_size=page_size, filter=filter, search=search, sorting=sorting
            )
            details = self._make_request(url, params=params)

            page_results = details.get("results", [])
            results.extend(result_class(**item) for item in page_results)

        return results

    @staticmethod
    def _build_query_params(
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        filter: Optional[str] = None,
        search: Optional[str] = None,
        sorting: Optional[str] = None,
    ) -> Dict[str, Any]:
        """
        Helper method to build query parameters for GET requests.

        :param page: The page number (optional).
        :param page_size: The number of items per page (optional).
        :param filter: An optional filter string.
        :param search: An optional search string.
        :param sorting: An optional sorting string.
        :return: A dictionary of query parameters.
        """
        params = {
            "filter": filter,
            "search": search,
            "sorting": sorting,
            "page": page,
            "pageSize": page_size,
        }
        return {k: v for k, v in params.items() if v is not None}

    def set_project_id(self, project_id: str) -> None:
        self.project_id = project_id

    def _validate_project_id(self) -> None:
        if not self.project_id:
            raise ValueError("Project ID is not set. Please set the project_id before making this request.")

    def _build_url_project_required(self, path: str) -> str:
        """Builds the URL that requires a project ID, using the client-specific base path."""
        self._validate_project_id()
        return f"{self.base_url}{self.base_path}{self.project_id}{path}"

    @property
    def base_path(self) -> str:
        """To be overridden by subclasses to provide the base path for each client."""
        raise NotImplementedError("Subclasses must define 'base_path'.")

    def get(self, endpoint: str, params: Optional[Dict] = None) -> Dict:
        url = f"{self.base_url}/{endpoint}"
        headers = {"Authorization": f"Bearer {self.auth_token}"}
        response = httpx.get(url, headers=headers, params=params)
        self._handle_response(response=response, url=url, method="GET")
        return response.json()

    def post(self, endpoint: str, data: Dict) -> Dict:
        url = f"{self.base_url}/{endpoint}"
        headers = {"Authorization": f"Bearer {self.auth_token}"}
        response = httpx.post(url, headers=headers, json=data)
        self._handle_response(response=response, url=url, method="POST")
        return response.json()

    def put(self, endpoint: str, data: Dict) -> Dict:
        url = f"{self.base_url}/{endpoint}"
        headers = {"Authorization": f"Bearer {self.auth_token}"}
        response = httpx.put(url, headers=headers, json=data)
        self._handle_response(response=response, url=url, method="PUT")
        return response.json()

    def delete(self, endpoint: str) -> Dict:
        url = f"{self.base_url}/{endpoint}"
        headers = {"Authorization": f"Bearer {self.auth_token}"}
        response = httpx.delete(url, headers=headers)
        self._handle_response(response=response, url=url, method="DELETE")
        return response.json()



from stellanow_api_internals.datatypes.workflow_mgmt import StellaEventDetailed

MOCK_RESPONSE_PAGE_1 = {
    "details": {
        "pageNumber": 1,
        "numberOfPages": 2,
        "pageSize": 1,
        "total": 2,
        "results": [
            {
                "id": "173fbe3d-04a4-47a1-a038-83262aa56a70",
                "projectId": "52b212f7-3da9-47eb-a2a8-adc9248dba74",
                "name": "electric_state_frequency",
                "isActive": True,
                "createdAt": "2024-03-06T18:22:19.335921Z",
                "updatedAt": "2024-06-27T08:28:40.973269Z",
            },
        ],
    },
    "status": "ok",
}

MOCK_RESPONSE_PAGE_2 = {
    "details": {
        "pageNumber": 2,
        "numberOfPages": 2,
        "pageSize": 1,
        "total": 2,
        "results": [
            {
                "id": "1f6d835e-2f01-468f-9501-04a6f7509389",
                "projectId": "52b212f7-3da9-47eb-a2a8-adc9248dba74",
                "name": "test_event_sdk",
                "isActive": True,
                "createdAt": "2024-04-09T11:51:55.966189Z",
                "updatedAt": "2024-07-22T15:51:32.276369Z",
            },
        ],
    },
    "status": "ok",
}

MOCK_EVENT_RESPONSE = {
    "details": {
        "id": "173fbe3d-04a4-47a1-a038-83262aa56a70",
        "name": "Test Event",
        "description": "Test event description",
        "projectId": "52b212f7-3da9-47eb-a2a8-adc9248dba74",
        "isActive": True,
        "createdAt": "2024-03-06T18:22:19.335921Z",
        "updatedAt": "2024-06-27T08:28:40.973269Z",
        "entities": [{"id": "entity1", "name": "Entity 1"}, {"id": "entity2", "name": "Entity 2"}],
        "fields": [
            {"id": "field1", "name": "Field 1", "fieldType": {"value": "string"}, "required": True},
            {"id": "field2", "name": "Field 2", "fieldType": {"value": "int"}, "required": False},
        ],
    },
    "status": "ok",
}


def test_get_event_details(mock_workflow_client):
    workflow_client = mock_workflow_client(MOCK_EVENT_RESPONSE)

    event_details: StellaEventDetailed = workflow_client.get_event_details(
        event_id="173fbe3d-04a4-47a1-a038-83262aa56a70"
    )

    assert event_details.id == "173fbe3d-04a4-47a1-a038-83262aa56a70"
    assert event_details.name == "Test Event", "Event name mismatch"
    assert event_details.description == "Test event description", "Event description mismatch"
    assert event_details.projectId == "52b212f7-3da9-47eb-a2a8-adc9248dba74", "Project ID mismatch"
    assert event_details.isActive is True, "isActive value mismatch"

    assert event_details.createdAt == "2024-03-06 18:22:19", "createdAt mismatch"
    assert event_details.updatedAt == "2024-06-27 08:28:40", "updatedAt mismatch"

    assert len(event_details.entities) == 2, "Entities count mismatch"
    assert event_details.entities[0].id == "entity1", "Entity 1 ID mismatch"
    assert event_details.entities[0].name == "Entity 1", "Entity 1 name mismatch"
    assert event_details.entities[1].id == "entity2", "Entity 2 ID mismatch"
    assert event_details.entities[1].name == "Entity 2", "Entity 2 name mismatch"

    assert len(event_details.fields) == 2, "Fields count mismatch"
    assert event_details.fields[0].id == "field1", "Field 1 ID mismatch"
    assert event_details.fields[0].name == "Field 1", "Field 1 name mismatch"
    assert event_details.fields[0].fieldType.value == "string", "Field 1 type mismatch"
    assert event_details.fields[0].required is True, "Field 1 required value mismatch"

    assert event_details.fields[1].id == "field2", "Field 2 ID mismatch"
    assert event_details.fields[1].name == "Field 2", "Field 2 name mismatch"
    assert event_details.fields[1].fieldType.value == "int", "Field 2 type mismatch"
    assert event_details.fields[1].required is False, "Field 2 required value mismatch"



"""
Copyright (C) 2022-2024 Stella Technologies (UK) Limited.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
"""

from uuid import uuid4

import pytest

from stellanow_api_internals.datatypes.workflow_mgmt import StellaProject, StellaWorkflowDetailed


def test_stella_project_with_none_archived():
    project = StellaProject(
        id=str(uuid4()),
        name="Test Project",
        organizationId=str(uuid4()),
        createdAt="2024-01-01T12:00:00Z",
        updatedAt="2024-01-02T12:00:00Z",
        archived=None,
    )

    assert project.archived is None
    assert project.name is "Test Project"


def test_stella_project_with_valid_archived():
    archived_date = "2024-01-03T12:00:00Z"
    project = StellaProject(
        id=str(uuid4()),
        name="Test Project",
        organizationId=str(uuid4()),
        createdAt="2024-01-01T12:00:00Z",
        updatedAt="2024-01-02T12:00:00Z",
        archived=archived_date,
    )

    assert project.archived == "2024-01-03 12:00:00"


def test_stella_project_without_archived():
    uuid = str(uuid4())
    project_dict = {
        "id": uuid,
        "name": "Test Project",
        "organizationId": uuid,
        "createdAt": "2024-01-01T12:00:00Z",
        "updatedAt": "2024-01-02T12:00:00Z",
    }
    project = StellaProject(**project_dict)

    assert project.id == uuid
    assert project.name == "Test Project"
    assert project.organizationId == uuid
    assert project.createdAt == "2024-01-01 12:00:00"
    assert project.updatedAt == "2024-01-02 12:00:00"


def test_stella_project_with_invalid_archived_format():
    with pytest.raises(ValueError):
        StellaProject(
            id=str(uuid4()),
            name="Test Project",
            organizationId=uuid4(),
            createdAt="2024-01-01T12:00:00Z",
            updatedAt="2024-01-02T12:00:00Z",
            archived="Invalid Date",
        )


def test_stella_workflow_detailed_with_none_activated_at():
    workflow = StellaWorkflowDetailed(
        id=str(uuid4()),
        name="Test Workflow",
        projectId=str(uuid4()),
        isActive=True,
        tags=["tag1", "tag2"],
        publishedVersion=1,
        createdAt="2024-01-01T12:00:00Z",
        updatedAt="2024-01-02T12:00:00Z",
        activatedAt=None,
        description="A test workflow",
        events=[],
        entities=[],
    )

    assert workflow.activatedAt is None


def test_stella_workflow_detailed_with_valid_activated_at():
    activated_at_date = "2024-01-03T12:00:00Z"
    workflow = StellaWorkflowDetailed(
        id=str(uuid4()),
        name="Test Workflow",
        projectId=str(uuid4()),
        isActive=True,
        tags=["tag1", "tag2"],
        publishedVersion=1,
        createdAt="2024-01-01T12:00:00Z",
        updatedAt="2024-01-02T12:00:00Z",
        activatedAt=activated_at_date,
        description="A test workflow",
        events=[],
        entities=[],
    )

    assert workflow.activatedAt == "2024-01-03 12:00:00"


def test_stella_workflow_detailed_with_invalid_activated_at_format():
    with pytest.raises(ValueError):
        StellaWorkflowDetailed(
            id=str(uuid4()),
            name="Test Workflow",
            projectId=str(uuid4()),
            isActive=True,
            tags=["tag1", "tag2"],
            publishedVersion=1,
            createdAt="2024-01-01T12:00:00Z",
            updatedAt="2024-01-02T12:00:00Z",
            activatedAt="Invalid Date",
            description="A test workflow",
            events=[],
            entities=[],
        )



"""
Copyright (C) 2022-2024 Stella Technologies (UK) Limited.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
"""

from datetime import datetime
from typing import List, Optional, Union

from pydantic import BaseModel, ConfigDict, RootModel

from stellanow_api_internals.core.helpers import StellaDateFormatter
from stellanow_api_internals.datatypes.core import (
    StellaBaseIdName,
    StellaBaseName,
    StellaExcludeNone,
    StellaFormattedDateTime,
)


class StellaBaseFieldType(BaseModel):
    value: str


class StellaPropagatedFrom(StellaBaseIdName):
    eventName: str


class StellaPropagatedFromList(RootModel):
    root: List[StellaPropagatedFrom]


class StellaPropagatedFromCreate(BaseModel):
    id: str


class StellaPropagatedFromCreateList(RootModel):
    root: List[StellaPropagatedFromCreate]


class StellaProject(StellaBaseIdName, StellaFormattedDateTime):
    organizationId: str
    archived: Optional[str] = None

    def __init__(self, **data):
        super().__init__(**data)
        if self.archived:
            self.archived = self.format_date(self.archived)


class StellaProjectDetailed(StellaProject):
    description: Optional[str] = None


class StellaProjectCreate(StellaBaseName):
    description: Optional[str] = None

    model_config = ConfigDict(extra="ignore")


class StellaModelFieldType(StellaBaseFieldType):
    modelRef: Optional[str] = None


class StellaModelField(BaseModel):
    id: str
    name: str
    fieldType: StellaModelFieldType


class StellaModelFieldList(RootModel):
    root: List[StellaModelField]


class StellaModelCreate(StellaBaseName):
    description: Optional[str] = None
    fields: StellaModelFieldList

    model_config = ConfigDict(extra="ignore")


class StellaEntityId(BaseModel):
    id: str


class StellaEntityIdList(RootModel):
    root: List[StellaEntityId]


class StellaShortEntity(BaseModel):
    id: str
    name: str


class StellaShortEntityList(RootModel):
    root: List[StellaShortEntity]


class StellaShortEvent(BaseModel):
    id: str
    name: str


class StellaShortEventList(RootModel):
    root: List[StellaShortEvent]


class StellaBaseEntity(StellaShortEntity, StellaFormattedDateTime):
    projectId: str
    isActive: bool


class StellaEntity(StellaBaseEntity):
    eventsCount: int


class StellaEntityField(StellaBaseIdName):
    fieldType: StellaBaseFieldType
    propagatedFrom: Optional[List[StellaPropagatedFrom]] = None


class StellaEntityFieldList(RootModel):
    root: List[StellaEntityField]


class StellaEntityFieldCreate(StellaBaseIdName):
    fieldType: StellaBaseFieldType


class StellaEntityFieldCreateList(RootModel):
    root: List[StellaEntityFieldCreate]


class StellaEntityFieldUpdate(BaseModel):
    name: Optional[str] = None
    fieldType: Optional[StellaBaseFieldType] = None
    propagatedFrom: Optional[List[str]] = None
    action: str = "update"
    id: Optional[str] = None


class StellaEntityFieldUpdateList(RootModel):
    root: List[StellaEntityFieldUpdate]


class StellaEntityDetailed(StellaBaseEntity):
    description: Optional[str] = None
    fields: Optional[List[StellaEntityField]] = None
    events: Optional[List[StellaShortEvent]] = None


class StellaEntityDetailedList(RootModel):
    root: List[StellaEntityDetailed]


class StellaEntityCreate(BaseModel):
    name: Optional[str] = None
    description: Optional[str] = None
    fields: Optional[StellaEntityFieldCreateList] = None

    model_config = ConfigDict(extra="ignore")


class StellaEntityUpdate(BaseModel):
    name: Optional[str] = None
    description: Optional[str] = None
    fields: Optional[StellaEntityFieldUpdateList] = None

    model_config = ConfigDict(extra="ignore")


class StellaFieldType(StellaBaseFieldType):
    modelRef: Optional[str] = None


class StellaSubField(BaseModel):
    id: str
    name: str
    fieldType: StellaFieldType
    required: bool
    path: List[str]
    modelFieldId: Optional[str] = None  # TODO I think this solve issue!


class StellaSubFieldCreate(BaseModel):
    modelFieldId: str
    required: bool


class StellaSubFieldList(RootModel):
    root: List[StellaSubField]


class StellaField(BaseModel):
    id: str
    name: str
    fieldType: StellaFieldType
    required: bool
    subfields: Optional[List[StellaSubField]] = None


class StellaFieldCreate(BaseModel):
    name: str
    fieldType: StellaFieldType
    required: bool
    subfields: Optional[List[StellaSubFieldCreate]] = None


class StellaFieldList(RootModel):
    root: List[StellaField]


class StellaEvent(StellaShortEvent, StellaFormattedDateTime):
    projectId: str
    isActive: bool


class StellaEventDetailed(StellaEvent):
    description: Optional[str] = None
    fields: List[StellaField]
    entities: List[StellaShortEntity]


class StellaEventDetailedList(RootModel):
    root: List[StellaEventDetailed]


class StellaEventCreate(StellaShortEvent):
    description: Optional[str] = None
    fields: List[StellaFieldCreate]
    entities: List[StellaShortEntity]


class StellaModel(StellaFormattedDateTime):
    id: str
    name: str


class StellaModelDetailed(StellaModel):
    description: Optional[str] = None
    fields: StellaModelFieldList


class StellaModelDetailedList(RootModel):
    root: List[StellaModelDetailed]


class StellaWorkflow(StellaBaseIdName, StellaFormattedDateTime):
    projectId: str
    tags: Optional[List[str]] = None
    publishedVersion: Optional[int] = None
    isActive: bool


class StellaWorkflowDetailed(StellaWorkflow):
    description: Optional[str] = None
    events: List[StellaShortEvent]
    entities: List[StellaShortEntity]
    activatedAt: Optional[str] = None

    def __init__(self, **data):
        super().__init__(**data)
        if self.activatedAt:
            self.activatedAt = self.format_date(self.activatedAt)


class StellaWorkflowDetailedList(RootModel):
    root: List[StellaWorkflowDetailed]


class StellaWorkflowCreate(StellaBaseName):
    description: Optional[str] = None
    tags: Optional[List[str]] = None
    entities: StellaEntityIdList


class StellaWorkflowDAGPosition(BaseModel):
    x: int
    y: int


class StellaWorkflowDAGDisplayInfo(BaseModel):
    title: str
    description: Optional[str] = None


class DAGNodeEventTypeFieldDefinition(BaseModel):
    type: Optional[str] = None
    id: str
    fieldId: str
    multiValue: bool
    eventName: str
    fieldName: str


class DAGNodeEntityTypeFieldDefinition(BaseModel):
    type: str
    id: str
    fieldId: str
    multiValue: bool
    entityName: str
    fieldName: str


class DAGNodeExplicitTypeFieldDefinition(BaseModel):
    id: str
    type: str
    value: str
    multiValue: bool


class DAGLimitedAggregationIntervalType(BaseModel):
    interval: str
    intervalStart: datetime
    intervalLength: int


class DAGNodeEventMetadataDefinition(BaseModel):
    id: str
    fieldName: str


class Condition(StellaExcludeNone):
    id: str
    fieldType: StellaBaseFieldType
    value: Optional[Union[bool, str, int, float, datetime, None]] = None
    left: Union[DAGNodeEventMetadataDefinition, DAGNodeEventTypeFieldDefinition, DAGNodeEntityTypeFieldDefinition]
    right: Union[DAGNodeEventTypeFieldDefinition, DAGNodeEntityTypeFieldDefinition, DAGNodeExplicitTypeFieldDefinition]
    condition: str
    negate: bool


class ConditionList(RootModel):
    root: List[Condition]


class DAGNodeWithCondition(StellaExcludeNone):
    id: str
    operator: str
    conditions: ConditionList


class DAGNodeEmpty(StellaExcludeNone):
    id: str


class DAGConditionNodeData(DAGNodeWithCondition):
    pass


class DAGFilterNodeData(DAGNodeWithCondition):
    pass


class DAGNotificationField(BaseModel):
    targetFieldName: str
    source: Union[DAGNodeEventTypeFieldDefinition, DAGNodeEntityTypeFieldDefinition, DAGNodeExplicitTypeFieldDefinition]


class DAGNotificationFieldList(RootModel):
    root: List[DAGNotificationField]


class DAGNotificationNodeData(StellaExcludeNone):
    id: str
    channel: str
    fields: DAGNotificationFieldList


class DAGPropagateToField(StellaExcludeNone):
    id: Optional[str] = None
    source: Optional[
        Union[DAGNodeEventTypeFieldDefinition, DAGNodeEntityTypeFieldDefinition, DAGNodeExplicitTypeFieldDefinition]
    ] = None
    targetFieldId: Optional[str] = None
    targetFieldName: Optional[str] = None


class DAGPropagateToFieldList(RootModel):
    root: List[DAGPropagateToField]


class DAGPropagateToEntityNodeData(StellaExcludeNone):
    id: str
    entityName: str
    fields: DAGPropagateToFieldList


class DAGClearEntityNodeData(DAGNodeEmpty):
    pass


class DAGPropagateToSourceNodeData(DAGNodeEmpty):
    pass


class DAGTerminationNodeData(DAGNodeEmpty):
    pass


class DAGTransformationFiled(StellaExcludeNone):
    function: str
    id: Optional[str] = None
    parameterName: Optional[str] = None
    functionParameters: Optional[List[str]] = None
    script: Optional[str] = None
    source: Optional[
        Union[DAGNodeEventTypeFieldDefinition, DAGNodeEntityTypeFieldDefinition, DAGNodeExplicitTypeFieldDefinition]
    ] = None
    targetFieldId: Optional[str] = None
    targetFieldName: Optional[str] = None


class DAGTransformationFiledList(RootModel):
    root: List[DAGTransformationFiled]


class DAGTransformationNodeData(StellaExcludeNone):
    id: str
    eventName: Optional[str] = None
    fields: DAGTransformationFiledList


class DAGAwaitNodeData(StellaExcludeNone):
    id: str
    eventName: str
    waitTime: int


class DAGAggregationNodeData(StellaExcludeNone):
    id: str
    aggregated: DAGNodeEventTypeFieldDefinition
    source: Union[DAGNodeEventTypeFieldDefinition, DAGNodeEntityTypeFieldDefinition]
    interval: Optional[str] = None
    intervalStart: Optional[str] = None
    intervalLength: Optional[int] = None


class StellaWorkflowDAGAggregated(BaseModel):
    id: str
    filedId: str
    multiValue: bool
    eventName: str
    filedName: str


class StellaWorkflowDAGEntityFieldDefinition(BaseModel):
    id: str
    fieldId: str
    multiValue: bool
    entityName: str
    fieldName: str


class StellaWorkflowDAGSource(BaseModel):
    EntityFieldDefinition: StellaWorkflowDAGEntityFieldDefinition


class DAGNodeConfig(StellaExcludeNone):
    id: str
    aggregationData: Optional[dict] = None
    clearEntityData: Optional[dict] = None
    conditionData: Optional[dict] = None
    filterData: Optional[dict] = None
    notificationData: Optional[dict] = None
    propagateToEntityData: Optional[dict] = None
    propagateToSourceData: Optional[dict] = None
    terminationData: Optional[dict] = None
    transformationData: Optional[dict] = None
    awaitData: Optional[dict] = None

    def to_dict(self):
        """Flatten the config and remove None values."""
        result = {"id": str(self.id)}
        for key, value in self.dict().items():
            if value is not None and key != "id":
                result.update(value)
        return result


class StellaWorkflowDAGData(StellaExcludeNone):
    displayInfo: StellaWorkflowDAGDisplayInfo
    config: Union[
        DAGAggregationNodeData,
        DAGClearEntityNodeData,
        DAGConditionNodeData,
        DAGFilterNodeData,
        DAGNotificationNodeData,
        DAGPropagateToEntityNodeData,
        DAGPropagateToSourceNodeData,
        DAGTerminationNodeData,
        DAGTransformationNodeData,
        DAGAwaitNodeData,
    ]
    isStateful: bool


class StellaWorkflowDAGNodes(StellaExcludeNone):
    id: str
    type: str
    position: StellaWorkflowDAGPosition
    data: StellaWorkflowDAGData


class StellaWorkflowDAGNodesList(RootModel):
    root: List[StellaWorkflowDAGNodes]


class StellaWorkflowDAGEdge(BaseModel):
    id: str
    source: str
    target: str


class StellaWorkflowDAGEdgeList(RootModel):
    root: List[StellaWorkflowDAGEdge]


class StellaWorkflowDAGStructure(StellaExcludeNone):
    nodes: StellaWorkflowDAGNodesList
    edges: StellaWorkflowDAGEdgeList


class StellaWorkflowDAG(BaseModel, StellaDateFormatter):
    id: str
    workflowId: str
    commitMessage: Optional[str] = None
    versionNumber: int
    isLatest: bool
    isPublished: bool
    createdAt: str
    structure: StellaWorkflowDAGStructure

    def __init__(self, **data):
        super().__init__(**data)
        self.createdAt = self.format_date(self.createdAt)


class StellaWorkflowDAGList(RootModel):
    root: List[StellaWorkflowDAG]


class StellaWorkflowDagCreate(BaseModel):
    commitMessage: Optional[str] = None
    structure: StellaWorkflowDAGStructure



"""
Copyright (C) 2022-2024 Stella Technologies (UK) Limited.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
"""

from typing import Dict, List, Optional

import httpx

from stellanow_api_internals.clients.base_api_client import StellanowBaseAPIClient
from stellanow_api_internals.core.enums import FilterIncludeInactive
from stellanow_api_internals.datatypes.notification_config import (
    StellaChannel,
    StellaChannelCreate,
    StellaChannelDetailed,
    StellaDestination,
    StellaDestinationCreate,
    StellaDestinationDetailed,
    StellaNotificationService,
    StellaNotificationServiceDetailed,
)

NOTIFICATION_CONFIG_API_BASE = "/notification-config/"
NOTIFICATION_CONFIG_API_PROJECT_CONTEXT = "/notification-config/projects/"


class NotificationConfigClient(StellanowBaseAPIClient):
    def __init__(
        self,
        base_url: str,
        username: str,
        password: str,
        organization_id: str,
        project_id: Optional[str] = None,
        client: Optional[httpx.Client] = None,
    ) -> None:
        super().__init__(base_url, username, password, organization_id, client)
        self.project_id = project_id or ""

    @property
    def base_path(self) -> str:
        return NOTIFICATION_CONFIG_API_PROJECT_CONTEXT

    @property
    def _services_url(self):
        return f"{self.base_url}{NOTIFICATION_CONFIG_API_BASE}services"

    @property
    def _service_url(self):
        return f"{self.base_url}{NOTIFICATION_CONFIG_API_BASE}services/{{serviceId}}"

    @property
    def _channels_url(self):
        return self._build_url_project_required("/channels")

    @property
    def _channel_url(self):
        return self._build_url_project_required("/channels/{channelId}")

    @property
    def _destinations_url(self):
        return self._build_url_project_required("/channels/{channelId}/destinations")

    @property
    def _destination_url(self):
        return self._build_url_project_required("/channels/{channelId}/destinations/{destinationId}")

    def create_channel(self, channel_data: Dict) -> StellaChannelDetailed:
        return self._create_or_update_resource(
            url=self._channels_url,
            data_model_class=StellaChannelCreate,
            return_class=StellaChannelDetailed,
            resource_data=channel_data,
        )

    def create_destination(self, destination_data: Dict, channel_id: str) -> StellaDestination:
        return self._create_or_update_resource(
            url=self._destinations_url.format(channelId=channel_id),
            data_model_class=StellaDestinationCreate,
            return_class=StellaDestination,
            resource_data=destination_data,
        )

    def get_services(
        self,
        page: int = 1,
        page_size: int = 20,
        include_inactive: Optional[bool] = None,
        search: Optional[str] = None,
        sorting: Optional[str] = "services:created:asc",
    ) -> List[StellaNotificationService]:
        filter_query = FilterIncludeInactive.INCLUDE_INACTIVE.value if include_inactive else None

        return self._get_list_resource(
            url=self._services_url,
            result_class=StellaNotificationService,
            page=page,
            page_size=page_size,
            filter=filter_query,
            search=search,
            sorting=sorting,
        )

    def get_service_details(self, service_id: str) -> StellaNotificationServiceDetailed:
        return self._get_resource(
            url=self._service_url.format(serviceId=service_id),
            result_class=StellaNotificationServiceDetailed,
        )

    def get_channels(
        self,
        page: int = 1,
        page_size: int = 20,
        include_inactive: Optional[bool] = None,
        search: Optional[str] = None,
        sorting: Optional[str] = "channels:created:asc",
    ) -> List[StellaChannel]:
        filter_query = FilterIncludeInactive.INCLUDE_INACTIVE.value if include_inactive else None

        return self._get_list_resource(
            url=self._channels_url,
            result_class=StellaChannel,
            page=page,
            page_size=page_size,
            filter=filter_query,
            search=search,
            sorting=sorting,
        )

    def get_channel_details(self, channel_id: str) -> StellaChannelDetailed:
        return self._get_resource(
            url=self._channel_url.format(channelId=channel_id), result_class=StellaChannelDetailed
        )

    def get_destinations(
        self,
        channel_id: str,
        page: int = 1,
        page_size: int = 20,
        include_inactive: Optional[bool] = None,
        search: Optional[str] = None,
        sorting: Optional[str] = "destinations:created:asc",
    ) -> List[StellaDestination]:
        filter_query = FilterIncludeInactive.INCLUDE_INACTIVE.value if include_inactive else None

        return self._get_list_resource(
            url=self._destinations_url.format(channelId=channel_id),
            result_class=StellaDestination,
            page=page,
            page_size=page_size,
            filter=filter_query,
            search=search,
            sorting=sorting,
        )

    def get_destination_details(self, channel_id: str, destination_id: str) -> StellaDestinationDetailed:
        return self._get_resource(
            url=self._destination_url.format(channelId=channel_id, destinationId=destination_id),
            result_class=StellaDestinationDetailed,
        )
